(***********************************************************************)
(*                                                                     *)
(*                              OCamlGen                               *)
(*                                                                     *)
(*                       OCaml module Generator                        *)
(*                                                                     *)
(*               EPI Pomdapi, INRIA Paris - Rocquencourt               *)
(*                                                                     *)
(*  Copyright 1997-2014 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: walk_print.ml,v 1.11 2014/01/31 09:51:53 fclement Exp $ *)

module Print : Action_spec.Make =
  functor (Io : Walk_spec.Out) ->

  struct

    let ppf = Io.buffer
    ;;

    let print_seq () =
      Format.fprintf ppf ";@ "
    ;;

    let meta_print_string s =
      Format.fprintf ppf
        "@[<1>Format.fprintf ppf@ \"%%,%s\"@]"
        (String.escaped s)
    ;;

    let quote_string ppf s =
      Format.fprintf ppf
        "\"%s\"" s
    ;;

    let sum_tag lab =
      Format.fprintf ppf
        "@[Format.fprintf ppf@ %a@ %a@]"
        quote_string "%s"
        quote_string lab
    ;;

    let begin_sum_inj lab =
      Format.fprintf ppf
        "@[Format.fprintf ppf@ %a@ %a;@ "
        quote_string "@[<1>%s@ "
        quote_string lab
    ;;
    let end_sum_inj _lab =
      print_seq ();
      meta_print_string "@]";
      Format.fprintf ppf "@]"
    ;;

    let begin_tuple () =
  (*    meta_print_string "@[<1>(wpbegin_tuple";*)
      meta_print_string "@[<1>(@,";
      print_seq ()
    ;;
      let begin_tuple_elem () = ()
      ;;
      let end_tuple_elem () = ()
      ;;
      let separator_tuple () =
        print_seq ();
        meta_print_string ",@ ";
        print_seq ()
      ;;
    let end_tuple () =
      print_seq ();
  (*    meta_print_string "@,wpend_tuple)@]"*)
      meta_print_string "@,)@]"
    ;;

    let begin_record_expr () =
      meta_print_string "@[<1>{@,"; print_seq ()
    ;;
    let end_record_expr () =
      meta_print_string "@,}@]"
    ;;

    let begin_record_field lab =
      meta_print_string ("@[<1>" ^ lab ^ " =@ "); print_seq ()
    ;;
    let end_record_field _lab =
      print_seq (); meta_print_string ";@]@ "; print_seq ()
    ;;

    let begin_application () =
      Format.fprintf ppf "%,@[<1>"
    ;;
      let separator_argument () = Format.fprintf ppf "%,@ "
      ;;
      let begin_argument () = ()
      ;;
      let end_argument () = ()
      ;;
    let end_application () =
      Format.fprintf ppf "%,@]"
    ;;

    let external_fun = function
      | "print_string"
      | "print_int"
      | "print_nativeint"
      | "print_int32"
      | "print_int64"
      | "print_float"
      | "print_char" as s ->
        Format.fprintf ppf "Lib_print.print_quoted_%s"
          (String.sub s 6 (String.length s - 6))

      | "print_unit"
      | "print_bool"
      | "print_fun"
      | "print_poly"
      | "print_abstract"
      | "print_external" as s ->
        Format.fprintf ppf "Lib_print.print_%s"
          (String.sub s 6 (String.length s - 6))

      | "print_list" -> Format.fprintf ppf "Lib_print.%s" "print_List"
      | "print_array" -> Format.fprintf ppf "Lib_print.%s" "print_Array"
      | "print_option" -> Format.fprintf ppf "Lib_print.%s" "print_Option"
      | "print_ref" -> Format.fprintf ppf "Lib_print.%s" "print_Ref"
      | s -> Format.fprintf ppf "%s" s
    ;;

    let begin_paren () =
  (*    meta_print_string "@[<1>(wpbegin_paren";*)
      meta_print_string "@[<1>("; print_seq ()
    ;;

    let end_paren () =
      print_seq ();
  (*    meta_print_string "@,wpend_paren)@]"*)
      meta_print_string "@,)@]"
    ;;

  end
;;

module Gen : Walk_spec.Make = Walk_gen.Gen;;

(* Generates a printing program ``a la'' Caml. *)
module Printer =
 functor (Io : Walk_spec.Out) ->
 Walker.Make (Gen) (Print) (Io)
;;
